#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const glob = require("glob");
const ply_1 = require("./ply");
const options_1 = require("./options");
const location_1 = require("./location");
const storage_1 = require("./storage");
const retrieval_1 = require("./retrieval");
const import_1 = require("./import");
const tsNode = require("ts-node");
const start = Date.now();
tsNode.register({ transpileOnly: true });
const opts = new options_1.Config(new options_1.Defaults(), true).options;
const { runOptions, ...options } = opts;
const plier = new ply_1.Plier(options);
if (runOptions === null || runOptions === void 0 ? void 0 : runOptions.import) {
    plier.logger.debug('Options', options);
    const importer = new import_1.Import(runOptions.import, options.testsLocation, options.prettyIndent, plier.logger);
    try {
        for (const path of options.args) {
            plier.logger.info('Importing', path);
            importer.doImport(new retrieval_1.Retrieval(path));
        }
    }
    catch (err) {
        plier.logger.error(err.message, err);
        process.exit(1);
    }
}
else {
    let paths = [];
    let args = options.args;
    const globOptions = {
        cwd: options.testsLocation,
        ignore: options.ignore
    };
    if (args && args.length > 0) {
        // make arg paths relative to tests loc
        if (options.testsLocation !== '.') {
            args = args.map(arg => {
                const argLoc = new location_1.Location(arg);
                if (argLoc.isChildOf(options.testsLocation)) {
                    return argLoc.relativeTo(options.testsLocation);
                }
                else {
                    plier.logger.error(`WARNING: ${arg} is not under testsLocation ${options.testsLocation}`);
                }
                return arg;
            });
        }
        for (const arg of args) {
            const hash = arg.indexOf('#');
            if (hash > 0) {
                paths.push(arg);
            }
            else {
                // treat as glob pattern
                for (const file of glob.sync(arg, globOptions)) {
                    paths.push(file);
                }
                if (paths.length === 0) {
                    throw new Error(`Files(s) not found: ${args}`);
                }
            }
        }
    }
    else {
        paths = [
            ...glob.sync(options.requestFiles, globOptions),
            ...glob.sync(options.caseFiles, globOptions),
            ...glob.sync(options.flowFiles, globOptions)
        ];
    }
    paths = paths.map(p => {
        return path.isAbsolute(p) ? p : options.testsLocation + path.sep + p;
    });
    plier.find(paths).then(plyees => {
        plier.logger.debug('Plyees', plyees);
        plier.run(plyees, runOptions).then(results => {
            const res = { Passed: 0, Failed: 0, Errored: 0, Pending: 0, Submitted: 0 };
            results.forEach(result => res[result.status]++);
            plier.logger.error('\nOverall Results: ' + JSON.stringify(res));
            plier.logger.info(`Overall Time: ${Date.now() - start} ms`);
            if (plier.options.outputFile) {
                new storage_1.Storage(plier.options.outputFile).write(JSON.stringify(res, null, plier.options.prettyIndent));
            }
            if (res.Failed || res.Errored) {
                process.exit(1);
            }
        }).catch(err => {
            plier.logger.error(err);
            process.exit(1);
        });
    }).catch(err => {
        plier.logger.error(err);
        process.exit(1);
    });
}
//# sourceMappingURL=cli.js.map