"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Postman = exports.Import = void 0;
const storage_1 = require("./storage");
const yaml = require("./yaml");
class Import {
    constructor(format, root, indent, logger) {
        this.format = format;
        this.root = root;
        this.indent = indent;
        this.logger = logger;
    }
    async doImport(retrieval) {
        switch (this.format) {
            case 'postman': {
                await new Postman(this.root, this.indent, this.logger).import(retrieval);
            }
        }
    }
}
exports.Import = Import;
class Postman {
    constructor(root, indent, logger) {
        this.root = root;
        this.indent = indent;
        this.logger = logger;
        this.storagePathToRequestsObj = new Map();
    }
    async import(from) {
        const contents = await from.read();
        if (!contents) {
            throw new Error(`Import source not found: ${from.location}`);
        }
        const obj = JSON.parse(contents);
        this.storagePathToRequestsObj.clear();
        if (obj.values) {
            // values
            const name = this.baseName(from.location, 'postman_environment');
            const values = {};
            for (const value of obj.values) {
                if (value.enabled) {
                    values[value.key] = value.value;
                }
            }
            this.writeStorage(`${this.root}/${name}.json`, JSON.stringify(values, null, this.indent));
        }
        else if (obj.item) {
            // requests
            const name = this.baseName(from.location, 'postman_collection');
            this.processItem(`${this.root}/${name}`, obj.item);
            for (const [path, requestsObj] of this.storagePathToRequestsObj) {
                this.writeStorage(path, yaml.dump(requestsObj, this.indent));
            }
        }
    }
    baseName(location, suffix) {
        let name = location.base;
        const dotPc = name.lastIndexOf(`.${suffix}`);
        if (dotPc > 0) {
            name = name.substring(0, dotPc);
        }
        return name;
    }
    writeStorage(path, content) {
        const storage = new storage_1.Storage(path);
        if (storage.exists) {
            this.logger.info(`Overwriting: ${storage}`);
        }
        else {
            this.logger.info(`Creating: ${storage}`);
        }
        storage.write(content);
    }
    processItem(path, item) {
        for (const it of item) {
            if (it.request) {
                // write the request
                const name = it.name;
                try {
                    const request = this.translateRequest(it.request);
                    // windows doesn't support : in file names
                    const storagePath = `${path.replace(/ \/ /g, '/').replace(/:/g, '-')}.ply.yaml`;
                    const reqsObj = this.storagePathToRequestsObj.get(storagePath);
                    if (reqsObj) {
                        reqsObj[name] = request;
                    }
                    else {
                        this.storagePathToRequestsObj.set(storagePath, { [name]: request });
                    }
                }
                catch (err) {
                    this.logger.error(`Request ${path}/${it.name} not imported due to: ${err.message}`);
                    this.logger.debug(err.message, err);
                }
            }
            if (it.item) {
                this.processItem(`${path}/${it.name}`, it.item);
            }
        }
    }
    translateRequest(postmanRequest) {
        const request = {
            url: this.replaceExpressions(postmanRequest.url.raw),
            method: this.replaceExpressions(postmanRequest.method)
        };
        if (postmanRequest.header && postmanRequest.header.length > 0) {
            request.headers = {};
            for (const head of postmanRequest.header) {
                request.headers[head.key] = this.replaceExpressions(head.value);
            }
        }
        if (postmanRequest.body) {
            if (typeof postmanRequest.body === 'string') {
                request.body = this.replaceExpressions(postmanRequest.body);
            }
            else {
                const mode = postmanRequest.body.mode;
                if (mode === 'graphql') {
                    request.body = this.replaceExpressions(postmanRequest.body.graphql.query);
                }
                else if (mode === 'raw') {
                    request.body = this.replaceExpressions(postmanRequest.body.raw);
                }
                else {
                    throw new Error(`Unsupported request body mode: ${postmanRequest.body.mode}`);
                }
            }
        }
        return request;
    }
    replaceExpressions(input) {
        return input.replace(/\{\{(.*?)}}/g, function (_a, b) {
            return '${' + b + '}';
        });
    }
}
exports.Postman = Postman;
//# sourceMappingURL=import.js.map