"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Config = exports.PLY_CONFIGS = exports.Defaults = void 0;
const findUp = require("find-up");
const yargs = require("yargs");
const retrieval_1 = require("./retrieval");
const yaml = require("./yaml");
/**
 * Locations are lazily inited to reflect bootstrapped testsLocation.
 */
class Defaults {
    constructor(testsLocation = '.') {
        this.testsLocation = testsLocation;
        this.requestFiles = '**/*.{ply.yaml,ply.yml}';
        this.caseFiles = '**/*.ply.ts';
        this.flowFiles = '**/*.flow';
        this.ignore = '**/{node_modules,bin,dist,out}/**';
        this.skip = '';
        this.resultFollowsRelativePath = true;
        this.valuesFiles = [];
        this.verbose = false;
        this.quiet = false;
        this.bail = false;
        this.parallel = false;
        this.batchRows = 1;
        this.batchDelay = 0;
        this.responseBodySortedKeys = true;
        this.genExcludeResponseHeaders = [
            'cache-control',
            'connection',
            'content-length',
            'date',
            'etag',
            'server',
            'transfer-encoding',
            'x-powered-by'
        ];
        this.prettyIndent = 2;
    }
    get expectedLocation() {
        if (!this._expectedLocation) {
            this._expectedLocation = this.testsLocation + '/results/expected';
        }
        return this._expectedLocation;
    }
    get actualLocation() {
        if (!this._actualLocation) {
            this._actualLocation = this.testsLocation + '/results/actual';
        }
        return this._actualLocation;
    }
    get logLocation() {
        if (!this._logLocation) {
            this._logLocation = this.actualLocation;
        }
        return this._logLocation;
    }
}
exports.Defaults = Defaults;
exports.PLY_CONFIGS = ['plyconfig.yaml', 'plyconfig.yml', 'plyconfig.json'];
class Config {
    constructor(defaults = new Defaults(), commandLine = false, configPath) {
        this.defaults = defaults;
        this.yargsOptions = {
            testsLocation: {
                describe: 'Tests base directory',
                alias: 't'
            },
            requestFiles: {
                describe: 'Request files glob pattern'
            },
            caseFiles: {
                describe: 'Case files glob pattern'
            },
            flowFiles: {
                describe: 'Flow files glob pattern'
            },
            ignore: {
                describe: 'File patterns to ignore'
            },
            skip: {
                describe: 'File patterns to skip'
            },
            submit: {
                describe: 'Send requests but don\'t verify results',
                alias: 's',
                type: 'boolean'
            },
            expectedLocation: {
                describe: 'Expected results base dir',
                type: 'string' // avoid premature reading of default
            },
            actualLocation: {
                describe: 'Actual results base dir',
                type: 'string' // avoid premature reading of default
            },
            resultFollowsRelativePath: {
                describe: 'Results under similar subpath'
            },
            logLocation: {
                describe: 'Test logs base dir',
                type: 'string' // avoid premature reading of default
            },
            valuesFiles: {
                describe: 'Values files (comma-separated)',
                type: 'string'
            },
            outputFile: {
                describe: 'Results summary json file path',
                type: 'string'
            },
            verbose: {
                describe: 'Much output (supersedes \'quiet\')'
            },
            quiet: {
                describe: 'Opposite of \'verbose\''
            },
            bail: {
                describe: 'Stop on first failure'
            },
            parallel: {
                describe: 'Run suites in parallel'
            },
            batchRows: {
                describe: '(Rowwise values) rows per batch'
            },
            batchDelay: {
                describe: '(Rowwise values) ms delay betw batches'
            },
            import: {
                describe: 'Import requests/values from (\'postman\')',
                type: 'string'
            },
            create: {
                describe: 'Create expected result from actual',
                type: 'boolean'
            },
            useDist: {
                describe: 'Load cases from dist instead of source',
                type: 'boolean'
            },
            responseBodySortedKeys: {
                describe: 'Sort response body JSON keys'
            },
            genExcludeResponseHeaders: {
                describe: 'Exclude from generated results',
                type: 'string'
            },
            prettyIndent: {
                describe: 'Format response JSON'
            }
        };
        this.options = this.load(defaults, commandLine, configPath);
        this.defaults.testsLocation = this.options.testsLocation;
        // result locations may need priming
        if (!this.options.expectedLocation) {
            this.options.expectedLocation = defaults.expectedLocation;
        }
        if (!this.options.actualLocation) {
            this.options.actualLocation = defaults.actualLocation;
        }
        if (!this.options.logLocation) {
            this.options.logLocation = this.options.actualLocation;
        }
    }
    load(defaults, commandLine, configPath) {
        let opts;
        if (commandLine) {
            // help pre-check to avoid premature yargs parsing
            const needsHelp = process.argv.length > 2 && process.argv[2] === '--help';
            if (!configPath && !needsHelp && yargs.argv.config) {
                configPath = '' + yargs.argv.config;
                console.debug(`Loading config from ${configPath}`);
            }
            if (!configPath) {
                configPath = findUp.sync(exports.PLY_CONFIGS, { cwd: defaults.testsLocation });
            }
            const config = configPath ? this.read(configPath) : {};
            let spec = yargs
                .usage('Usage: $0 <tests> [options]')
                .help('help').alias('help', 'h')
                .version().alias('version', 'v')
                .config(config)
                .option('config', { description: 'Ply config location', type: 'string', alias: 'c' });
            for (const option of Object.keys(this.yargsOptions)) {
                const yargsOption = this.yargsOptions[option];
                let type = yargsOption.type;
                if (!type) {
                    // infer from default
                    type = typeof defaults[option];
                }
                spec = spec.option(option, {
                    type,
                    // default: val, // clutters help output
                    ...yargsOption
                });
                if (type === 'boolean') {
                    spec = spec.boolean(option);
                }
            }
            opts = spec.argv;
            if (typeof opts.valuesFiles === 'string') {
                opts.valuesFiles = opts.valuesFiles.split(',').map((v) => v.trim());
            }
            if (opts.genExcludeResponseHeaders) {
                if (typeof opts.genExcludeResponseHeaders === 'string') {
                    opts.genExcludeResponseHeaders = opts.genExcludeResponseHeaders.split(',').map((v) => v.trim());
                }
                opts.genExcludeResponseHeaders = opts.genExcludeResponseHeaders.map((v) => v.toLowerCase());
            }
            opts.args = opts._;
            delete opts._;
        }
        else {
            if (!configPath) {
                configPath = findUp.sync(exports.PLY_CONFIGS, { cwd: defaults.testsLocation });
            }
            opts = configPath ? this.read(configPath) : {};
        }
        let options = { ...defaults, ...opts };
        // clean up garbage keys added by yargs, and private defaults
        options = Object.keys(options).reduce((obj, key) => {
            if (key.length > 1 && key.indexOf('_') === -1 && key.indexOf('-') === -1) {
                obj[key] = options[key];
            }
            return obj;
        }, {});
        // run options
        options.runOptions = {};
        if (options.submit) {
            options.runOptions.submit = options.submit;
            delete options.submit;
        }
        if (options.create) {
            options.runOptions.createExpected = options.create;
            delete options.create;
        }
        if (options.useDist) {
            options.runOptions.useDist = options.useDist;
            delete options.useDist;
        }
        if (options.import) {
            options.runOptions.import = options.import;
            delete options.import;
        }
        return options;
    }
    read(configPath) {
        const retrieval = new retrieval_1.Retrieval(configPath);
        const contents = retrieval.sync();
        if (typeof contents === 'string') {
            if (retrieval.location.isYaml) {
                return yaml.load(retrieval.location.path, contents);
            }
            else {
                return JSON.parse(contents);
            }
        }
        else {
            throw new Error("Cannot load config: " + configPath);
        }
    }
}
exports.Config = Config;
//# sourceMappingURL=options.js.map