"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlyRequest = void 0;
const cross_fetch_1 = require("cross-fetch");
const response_1 = require("./response");
const logger_1 = require("./logger");
const result_1 = require("./result");
const form_1 = require("./form");
const util = require("./util");
const subst = require("./subst");
const names_1 = require("./names");
class PlyRequest {
    /**
     * @param name test name
     * @param obj object to parse for contents
     */
    constructor(name, obj, logger, retrieval) {
        this.name = name;
        this.logger = logger;
        this.type = 'request';
        if (!obj.url) {
            throw new Error(`Request ${name} in ${retrieval} is missing 'url'`);
        }
        this.url = obj.url.trim();
        if (!obj.method) {
            throw new Error(`Request ${name} in ${retrieval} is missing 'method'`);
        }
        this.method = obj.method.trim();
        this.headers = obj.headers || {};
        this.body = obj.body;
        this.start = obj.start || 0;
        this.end = obj.end;
    }
    getSupportedMethod(method) {
        const upperCase = method.toUpperCase().trim();
        if (upperCase === 'GET'
            || upperCase === 'HEAD'
            || upperCase === 'POST'
            || upperCase === 'PUT'
            || upperCase === 'DELETE'
            || upperCase === 'CONNECT'
            || upperCase === 'OPTIONS'
            || upperCase === 'TRACE'
            || upperCase === 'PATCH') {
            return upperCase;
        }
    }
    get isGraphQl() {
        if (this.body) {
            return this.body.startsWith('query')
                || this.body.startsWith('mutation');
        }
        return false;
    }
    getRunId(values) {
        return values[names_1.RUN_ID];
    }
    /**
     * Call submit() to send the request without producing actual results
     * or comparing with expected.  Useful for cleaning up or restoring
     * REST resources before/after testing (see Case.before()/after()).
     */
    async submit(values, options, runOptions) {
        return await this.doSubmit(this.getRunId(values), this.getRequest(values, options, true), runOptions);
    }
    async doSubmit(runId, requestObj, runOptions) {
        const logLevel = (runOptions === null || runOptions === void 0 ? void 0 : runOptions.submit) ? logger_1.LogLevel.info : logger_1.LogLevel.debug;
        const before = new Date().getTime();
        const { Authorization: _auth, ...loggedHeaders } = requestObj.headers;
        const loggedRequest = { ...requestObj, runId, headers: loggedHeaders };
        this.logger.log(logLevel, 'Request', loggedRequest);
        const ctHeader = util.header(requestObj.headers, 'content-type');
        if (ctHeader && ctHeader[1].startsWith('multipart/form-data')) {
            requestObj = new form_1.MultipartForm(requestObj).getRequest();
        }
        const { url: _url, ...fetchRequest } = requestObj;
        const response = await cross_fetch_1.default(requestObj.url, fetchRequest);
        const status = { code: response.status, message: response.statusText };
        const headers = this.responseHeaders(response.headers);
        const body = await response.text();
        const time = new Date().getTime() - before;
        const plyResponse = new response_1.PlyResponse(runId, status, headers, body, time);
        this.logger.log(logLevel, 'Response', plyResponse);
        return plyResponse;
    }
    /**
     * Request object with substituted values
     */
    getRequest(values, options, includeAuthHeader = false) {
        const url = subst.replace(this.url, values, this.logger);
        if (!url.startsWith('http://') && !url.startsWith('https://')) {
            throw new Error('Invalid url: ' + url);
        }
        const method = subst.replace(this.method, values, this.logger).toUpperCase();
        if (!this.getSupportedMethod(method)) {
            throw new Error('Unsupported method: ' + method);
        }
        const headers = {};
        for (const key of Object.keys(this.headers)) {
            headers[key] = subst.replace(this.headers[key], values, this.logger);
        }
        if (!includeAuthHeader) {
            delete headers.Authorization;
        }
        let body = this.body;
        if (body) {
            body = subst.replace(body, values, this.logger);
            if (this.isGraphQl) {
                // graphql
                this.graphQl = body;
                body = JSON.stringify({ query: body }, null, options === null || options === void 0 ? void 0 : options.prettyIndent);
            }
        }
        return {
            name: this.name,
            type: this.type,
            url,
            method,
            headers,
            body,
            submitted: this.submitted,
            submit: () => { throw new Error('Not implemented'); }
        };
    }
    responseHeaders(headers) {
        const obj = {};
        headers.forEach((value, name) => {
            obj[name] = value;
        });
        return obj;
    }
    /**
     * Only to be called in the context of a Suite (hence 'runtime').
     * To execute a test programmatically, call one of the Suite.run() overloads.
     * Or to send a request without testing, call submit().
     * @returns result with request invocation and status of 'Pending'
     */
    async run(runtime, values, runOptions) {
        var _a;
        this.submitted = new Date();
        const requestObject = this.getRequest(values, runtime.options, true);
        const id = this.logger.level === logger_1.LogLevel.debug ? ` (${this.getRunId(values)})` : '';
        this.logger.info(`Request '${this.name}'${id} submitted at ${util.timestamp(this.submitted, this.logger.level === logger_1.LogLevel.debug)}`);
        const runOpts = { ...runOptions };
        const expectedExists = await runtime.results.expected.exists;
        if ((runOptions === null || runOptions === void 0 ? void 0 : runOptions.submitIfExpectedMissing) && !expectedExists) {
            runOpts.submit = true;
        }
        const runId = this.getRunId(values);
        const response = await this.doSubmit(runId, requestObject, runOpts);
        if (response.headers && ((runOptions === null || runOptions === void 0 ? void 0 : runOptions.createExpected) || (runOptions === null || runOptions === void 0 ? void 0 : runOptions.createExpectedIfMissing) && !expectedExists) && ((_a = runtime.options.genExcludeResponseHeaders) === null || _a === void 0 ? void 0 : _a.length)) {
            for (const key of Object.keys(response.headers)) {
                if (runtime.options.genExcludeResponseHeaders.includes(key)) {
                    delete response.headers[key];
                }
            }
        }
        const result = new result_1.PlyResult(this.name, requestObject, response.getResponse(runId, runtime.options, runtime.responseHeaders, true));
        if (this.graphQl) {
            result.graphQl = this.graphQl;
        }
        return result;
    }
}
exports.PlyRequest = PlyRequest;
//# sourceMappingURL=request.js.map