"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlyStep = exports.getStepId = void 0;
const logger_1 = require("./logger");
const request_1 = require("./request");
const subst = require("./subst");
const util = require("./util");
const yaml = require("./yaml");
function getStepId(step) {
    return step.subflow ? `${step.subflow.id}.${step.step.id}` : step.step.id;
}
exports.getStepId = getStepId;
class PlyStep {
    constructor(step, requestSuite, logger, flowPath, flowInstanceId, subflow) {
        this.step = step;
        this.requestSuite = requestSuite;
        this.logger = logger;
        this.flowPath = flowPath;
        this.subflow = subflow;
        this.type = 'flow';
        this.name = getStepId(this);
        this.stepName = step.name.replace(/\r?\n/g, ' ');
        this.instance = {
            id: util.genId(),
            flowInstanceId,
            stepId: step.id,
            status: 'In Progress'
        };
    }
    async run(runtime, values, runOptions) {
        var _a, _b, _c, _d, _e, _f;
        this.instance.start = new Date();
        let result;
        let stepRes;
        const level = this.subflow ? 1 : 0;
        try {
            runtime.appendResult(`${this.stepName}:`, level, runOptions === null || runOptions === void 0 ? void 0 : runOptions.createExpected, util.timestamp(this.instance.start));
            runtime.appendResult(`id: ${this.step.id}`, level + 1, runOptions === null || runOptions === void 0 ? void 0 : runOptions.createExpected);
            if (this.subflow && this.step.path === 'start' && !(runOptions === null || runOptions === void 0 ? void 0 : runOptions.submit) && !(runOptions === null || runOptions === void 0 ? void 0 : runOptions.createExpected)) {
                this.padActualStart(this.subflow.id);
            }
            else if (this.step.path === 'stop' && !this.subflow) {
                let name = this.flowPath;
                const lastSlash = name.lastIndexOf('/');
                if (lastSlash > 0 && lastSlash < name.length - 1)
                    name = name.substring(lastSlash + 1);
                const runId = this.logger.level === logger_1.LogLevel.debug ? ` (${this.instance.flowInstanceId})` : '';
                this.logger.info(`Finished flow: ${name}${runId}`);
            }
            else if (this.step.path === 'request') {
                let url = (_a = this.step.attributes) === null || _a === void 0 ? void 0 : _a.url;
                if (!url)
                    throw new Error('Missing attribute: url');
                url = subst.replace(url, values, this.logger);
                let method = (_b = this.step.attributes) === null || _b === void 0 ? void 0 : _b.method;
                if (!method)
                    throw new Error('Missing attribute: method');
                method = subst.replace(method, values, this.logger);
                const headers = {};
                if ((_c = this.step.attributes) === null || _c === void 0 ? void 0 : _c.headers) {
                    const rows = JSON.parse(this.step.attributes.headers);
                    for (const row of rows) {
                        headers[row[0]] = subst.replace(row[1], values, this.logger);
                    }
                }
                let body = (_d = this.step.attributes) === null || _d === void 0 ? void 0 : _d.body;
                if (body) {
                    body = subst.replace(body, values, this.logger);
                }
                const requestObj = {
                    name: this.name,
                    type: 'request',
                    url,
                    method,
                    headers,
                    body,
                    submitted: new Date(),
                    submit: (_values) => { throw new Error('Not implemented'); }
                };
                const request = new request_1.PlyRequest(this.name, requestObj, this.logger, runtime.retrieval);
                if (request.isGraphQl) {
                    request.graphQl = body;
                    body = JSON.stringify({ query: body }, null, (_e = runtime.options) === null || _e === void 0 ? void 0 : _e.prettyIndent);
                }
                this.instance.data = {
                    request: yaml.dump(request.getRequest(values, runtime.options), runtime.options.prettyIndent)
                };
                if (((_f = this.step.attributes) === null || _f === void 0 ? void 0 : _f.submit) === 'true') {
                    const response = await request.submit(values, runtime.options, { ...runOptions, submit: true });
                    this.instance.data.response = yaml.dump(response, runtime.options.prettyIndent);
                }
                else {
                    this.requestSuite.tests[this.name] = request;
                    const result = await this.requestSuite.run(this.name, values, runOptions);
                    if (result.status !== 'Passed' && result.status !== 'Submitted') {
                        this.instance.status = result.status === 'Failed' ? 'Failed' : 'Errored';
                        this.instance.message = result.message;
                    }
                    if (result.response) {
                        let response = result.response;
                        if (result.response.body) {
                            // convert from object
                            response = { ...result.response, body: JSON.stringify(result.response.body, null, runtime.options.prettyIndent) };
                        }
                        this.instance.data.response = yaml.dump(response, runtime.options.prettyIndent);
                    }
                }
            }
            if (this.instance.status === 'In Progress') { // not overwritten by step execution
                this.instance.status = 'Completed';
            }
            this.instance.end = new Date();
            if (!(runOptions === null || runOptions === void 0 ? void 0 : runOptions.submit) && !(runOptions === null || runOptions === void 0 ? void 0 : runOptions.createExpected)) {
                await this.padActualStart(this.name);
            }
            result = this.mapResult(runOptions);
        }
        catch (err) {
            this.logger.error(err.message, err);
            this.instance.status = 'Errored';
            this.instance.message = err.message;
            result = { name: this.stepName, status: 'Errored', message: this.instance.message || '' };
        }
        // append status, result and message to actual result
        if (this.instance.end) {
            const elapsed = this.instance.end.getTime() - this.instance.start.getTime();
            runtime.appendResult(`status: ${this.instance.status}`, level + 1, runOptions === null || runOptions === void 0 ? void 0 : runOptions.createExpected, `${elapsed} ms`);
            if (typeof stepRes === 'boolean' || typeof stepRes === 'number' || stepRes) {
                this.instance.result = '' + stepRes;
                runtime.appendResult(`result: ${this.instance.result}`, level + 1, runOptions === null || runOptions === void 0 ? void 0 : runOptions.createExpected);
            }
            if (this.instance.message) {
                runtime.appendResult(`message: '${this.instance.message}'`, level + 1, runOptions === null || runOptions === void 0 ? void 0 : runOptions.createExpected);
            }
        }
        return result;
    }
    async padActualStart(name) {
        const expectedYaml = await this.requestSuite.runtime.results.getExpectedYaml(name);
        if (expectedYaml.start > 0) {
            const actualYaml = this.requestSuite.runtime.results.getActualYaml(name);
            if (expectedYaml.start > actualYaml.start) {
                this.requestSuite.runtime.results.actual.padLines(actualYaml.start, expectedYaml.start - actualYaml.start);
            }
        }
    }
    /**
     * Maps instance status to ply result
     */
    mapResult(runOptions) {
        let resultStatus;
        if (this.instance.status === 'In Progress' || this.instance.status === 'Waiting') {
            resultStatus = 'Pending';
        }
        else if (this.instance.status === 'Completed' || this.instance.status === 'Canceled') {
            resultStatus = (runOptions === null || runOptions === void 0 ? void 0 : runOptions.submit) ? 'Submitted' : 'Passed';
        }
        else {
            resultStatus = this.instance.status;
        }
        return { name: this.stepName, status: resultStatus, message: this.instance.message || '' };
    }
}
exports.PlyStep = PlyStep;
//# sourceMappingURL=step.js.map