"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.header = exports.fwdSlashes = exports.lineComment = exports.lines = exports.newlines = exports.timeparse = exports.genId = exports.nanoTime = exports.timestamp = exports.locale = void 0;
const process = require("process");
const osLocale = require("os-locale");
function locale() {
    let locale = osLocale.sync();
    if (!locale || locale.toLocaleLowerCase() === 'c') {
        locale = 'en-US';
    }
    return locale;
}
exports.locale = locale;
/**
 * Turn a date into a timestamp based on the OS locale
 * Note: Node/V8 bug causes hour = 24 (https://github.com/nodejs/node/issues/33089)
 * eg: 7/3/2021, 24:52:33:347
 */
function timestamp(date, withTimeZone = false) {
    const millis = String(date.getMilliseconds()).padStart(3, '0');
    const tz = withTimeZone ? date.toTimeString().substring(date.toTimeString().indexOf(' ')) : '';
    return `${date.toLocaleString(locale(), { hour12: false })}:${millis}${tz}`;
}
exports.timestamp = timestamp;
/**
 * time in nanoseconds
 */
function nanoTime() {
    const hrTime = process.hrtime();
    return hrTime[0] * 1000000000 + hrTime[1];
}
exports.nanoTime = nanoTime;
function genId() {
    return nanoTime().toString(16);
}
exports.genId = genId;
/**
 * Expected format is per timestamp() --
 * TODO: Only works for these two locale formats:
 *     MM/DD/YYYY, HH:mm:ss
 *     DD/MM/YYYY, HH:mm:ss (not tested actually)
 */
function timeparse(time) {
    const parser = /(\d+?)\/(\d+?)\/(\d+?), (\d+?):(\d+?):(\d+?):(\d+?)$/;
    const match = time.match(parser);
    if (match) {
        const formatObj = new Intl.DateTimeFormat(locale()).formatToParts(new Date());
        const first = formatObj[Object.keys(formatObj)[0]].type;
        const month = first === 'month' ? 1 : 2;
        const day = month === 1 ? 2 : 1;
        return new Date(parseInt(match[3]), // year
        parseInt(match[month]) - 1, // monthIndex
        parseInt(match[day]), // day
        parseInt(match[4]), // hours
        parseInt(match[5]), // minutes
        parseInt(match[6]), // seconds
        parseInt(match[7]) // millis
        );
    }
}
exports.timeparse = timeparse;
/**
 * Remove windows newline characters (\r)
 */
function newlines(input) {
    return input.replace(/\r/g, '');
}
exports.newlines = newlines;
/**
 * split a string into an array of lines, ignoring escaped
 */
function lines(input) {
    return input.split(/\r?\n/);
}
exports.lines = lines;
/**
 * Return the trailing comment portion of a line. Only works with no
 * embedded comment tokens (in string content, etc).
 */
function lineComment(line, token = '#') {
    const hash = line.indexOf(token);
    if (hash >= 0 && hash < line.length - 1) {
        return line.substr(hash + 1).trim();
    }
}
exports.lineComment = lineComment;
/**
 * substitute a forward slash for all backslashes
 */
function fwdSlashes(path) {
    return path.replace(/\\/g, '/');
}
exports.fwdSlashes = fwdSlashes;
function header(headers, name) {
    const key = name.toLowerCase();
    const match = Object.keys(headers).find(h => h.toLowerCase() === key);
    if (match)
        return [match, headers[match]];
}
exports.header = header;
//# sourceMappingURL=util.js.map